"""Code I am not proud of 🤗"""

import re, typing, glob, pathlib, contextlib, sys


class ContextDepth:
    """Count the current depth of a context manager invocation."""

    depth = 0

    def __enter__(self):
        self.depth += 1

    def __exit__(self, *e):
        self.depth -= 1


def html_comment(text):
    return f"""<!--\n{text}\n\n-->"""


def istype(object, cls):
    return isinstance(object, type) and issubclass(object, cls)


(FENCE, CONTINUATION, SEMI, COLON, MAGIC, DOCTEST), QUOTES, SPACE = (
    "``` \\ ; : %% >>>".split(),
    ('"""', "'''"),
    " ",
)
WHITESPACE = re.compile("^\s*", re.MULTILINE)


def indents(str: str) -> int:
    return len(str) - len(str.lstrip())


def num_first_indent(text: str) -> int:
    """The number of indents for the first blank line."""
    for str in text.splitlines():
        if str.strip():
            return indents(str)
    return 0


def num_last_indent(text: str) -> int:
    """The number of indents for the last blank line."""
    for str in reversed(text.splitlines()):
        if str.strip():
            return indents(str)
    return 0


def base_indent(tokens: typing.List[dict]) -> int:
    "Peek into mistune tokens and find the last code indent."
    for i, token in enumerate(tokens):
        if token["type"] == "code":
            code = token["text"]
            if code.lstrip().startswith(FENCE):
                continue
            indent = num_first_indent(code)
            break
    else:
        indent = 4
    return indent


def quote(text: str) -> str:
    """Wrap strings in triple quoted block strings."""
    if text.strip():
        left, right = len(text) - len(text.lstrip()), len(text.rstrip())
        quote = QUOTES[(text[right - 1] in QUOTES[0]) or (QUOTES[0] in text)]
        return text[:left] + quote + text[left:right] + quote + text[right:]
    return text


def whiten(text: str) -> str:
    """`whiten` strips empty lines because the `markdown.BlockLexer` doesn't like that."""
    return "\n".join(x.rstrip() for x in text.splitlines())


def strip_front_matter(text: str, sep=None) -> str:
    """Remove yaml front matter froma string."""
    if text.startswith("---\n"):
        front_matter, sep, rest = text[4:].partition("\n---")
    if sep:
        return "".join(rest.splitlines(True)[1:])
    return text


def ansify(str: str, format="md"):
    """High source to be printed in the terms."""
    import pygments.formatters.terminal256

    return pygments.highlight(
        str,
        pygments.lexers.find_lexer_class_by_name(format)(),
        pygments.formatters.terminal256.Terminal256Formatter(),
    )


def yield_files(files: typing.Sequence[str], recursive=False) -> typing.Generator:
    """Return a list of files from a collection of files and globs."""
    for file in files:
        yield from map(
            pathlib.Path,
            getattr(glob, ["", "r"][recursive] + "glob")(str(file))
            if "*" in str(file)
            else [file],
        )


@contextlib.contextmanager
def argv(*args: str):
    argv = sys.argv
    if len(args) == 1 and isinstance(args[0], str):
        args = args[0].split()
    sys.argv = list(args)
    yield
    sys.argv = list(argv)


def strip_shebang(str):
    return re.sub(re.compile(r"#!/.+\n"), "", str)


def strip_html_comment(str):
    return re.sub("(<!--[\s\S]*-->?)", "", str)


def strip_front_matter(text: str, sep=None) -> str:
    """Remove yaml front matter froma string."""
    if text.startswith("---\n"):
        front_matter, sep, rest = text[4:].partition("\n---")
    if sep:
        return "".join(rest.splitlines(True)[1:])
    return text


@contextlib.contextmanager
def sys_path():
    root = "." in sys.path
    if root:
        sys.path = ["."] + sys.path
    yield
    if root:
        sys.path.pop(sys.path.index("."))


def pidgy_builtins():
    import IPython, toolz, poser

    return {
        **{
            k: v
            for k, v in vars(IPython.display).items()
            if istype(v, IPython.core.display.DisplayObject)
        },
        **toolz.valfilter(callable, vars(toolz)),
        "shell": IPython.get_ipython(),
        "λ": poser.λ,
        "Λ": poser.Λ,
    }


def clean_doctest_traceback(str, *lines):
    *_, str = str.partition("-" * 70)
    return str.lstrip()



import pidgy

with pidgy.pidgyLoader(lazy=True):
    try:
        from . import intro, best_practices
    except:
        import intro, best_practices
main = __name__ == "__main__"
with pidgy.pidgyLoader(main=main, lazy=not main):
    try:
        from . import readme
    except:
        import readme



import pluggy

specification, implementation = (
    pluggy.HookspecMarker("pidgy"),
    pluggy.HookimplMarker("pidgy"),
)
with __import__("importnb").Notebook(lazy=True):
    try:
        from . import loader, tangle
        from .loader import pidgyLoader
    except:
        import loader, tangle
        from loader import pidgyLoader

with pidgyLoader(lazy=True):
    try:
        from . import weave, testing, magic, runpidgy
        from .shell import load_ipython_extension, unload_ipython_extension
    except:
        import weave, testing, magic, runpidgy
        from shell import load_ipython_extension, unload_ipython_extension

if __import__("IPython").get_ipython():
    magic.load_ipython_extension(__import__("IPython").get_ipython())



import inspect, click, stringcase, enum, uuid, datetime, pathlib, typing, types, builtins


def autoclick(
    *object: typing.Union[types.FunctionType, click.Command], group=None, **settings
) -> click.Command:
    """Automatically generate a click command line application using type inference."""
    app = group or click.Group()
    for command in object:
        if isinstance(command, click.Command):
            app.add_command(command)
        elif isinstance(command, types.FunctionType):
            decorators = signature_to_decorators(command)
            command = command_from_decorators(
                command, *decorators, **settings, help=inspect.getdoc(command)
            )
            app.add_command(command)
        elif isinstance(command, dict):
            decorators = decorators_from_dict(command)
            command = command_from_decorators(None, *decorators, **settings)
    return command if len(object) == 1 else app


def istype(x: typing.Any, y: type) -> bool:
    if isinstance(x, type):
        return issubclass(x, y)
    return False


def click_type(
    object: typing.Union[type, tuple], default=None
) -> typing.Union[type, click.types.ParamType]:
    """Translate python types to click's subset of types."""
    if isinstance(object, typing._GenericAlias):
        return click_type(object.__args__[0], default)
    elif isinstance(object, type):
        if issubclass(object, datetime.datetime):
            return click.DateTime()
        if issubclass(object, typing.Tuple):
            return click.Tuple(object.__args__)
        if issubclass(object, uuid.UUID):
            return click.UUID(default)
        if object is list:
            return
        if issubclass(object, set):
            return click.Choice(object)

        if issubclass(object, pathlib.Path):
            return click.Path()
        if object in {builtins.object, typing.Any}:
            return
        return object
    else:
        if isinstance(object, tuple):
            if all(isinstance(x, int) for x in object[:2]):
                return click.IntRange(*object)
            if all(isinstance(x, float) for x in object[:2]):
                return click.FloatRange(*object)


def command_from_decorators(command, *decorators, **settings):
    if command is None:
        *decorators, command = decorators
    for decorator in reversed(decorators):
        command = decorator(command)
    return click.command(no_args_is_help=bool(decorators), **settings)(command)


def decorators_from_dicts(annotations, defaults, *decorators):
    for k, v in annotations.items():
        if k in defaults:
            t = click_type(v, defaults.get(k))
            decorators += (
                click.option(
                    "-" * (1 if len(k) == 1 else 2) + stringcase.spinalcase(k),
                    type=t,
                    default=defaults.get(k),
                    show_default=True,
                    is_flag=v is bool,
                ),
            )

        elif isinstance(v, typing._GenericAlias) or istype(v, list):
            decorators += (
                click.argument(
                    stringcase.spinalcase(k),
                    type=click_type(getattr(v, "__args__", (str,))[0]),
                    nargs=-1,
                ),
            )
        else:
            decorators += (
                click.argument(stringcase.spinalcase(k), type=click_type(v)),
            )
    return decorators


def decorators_from_dict(object):
    return decorators_from_dicts(object.get("__annotations__", {}), object)


def decorators_from_module(object):
    return decorators_from_dict(vars(object))


def signature_to_decorators(object, *decorators):
    signature = inspect.signature(object)
    decorators += decorators_from_dicts(
        {
            k: typing.List[v.annotation]
            if v.kind == inspect._ParameterKind.VAR_POSITIONAL
            else v.annotation
            for k, v in signature.parameters.items()
            if k != "ctx"
        },
        {
            k: v.default
            for k, v in signature.parameters.items()
            if v.default != inspect._empty
        },
    )
    for k, v in signature.parameters.items():
        if k == "ctx":
            decorators += (click.pass_context,)
        break
    return decorators



from pathlib import Path
import setuptools, datetime

name = "pidgy"

__version__ = None

here = Path(__file__).parent

setup_args = dict(
    name=name,
    version=datetime.datetime.now().strftime("%Y.%m.%d"),
    author="deathbeds",
    author_email="tony.fast@gmail.com",
    description="Literate computing for literate programming",
    long_description=((here / "README.md").read_text() + "\n\n"),
    long_description_content_type="text/markdown",
    url="https://github.com/deathbeds/pidgy",
    python_requires=">=3.6",
    license="BSD-3-Clause",
    setup_requires=["pytest-runner"],
    tests_require=["pytest", "hypothesis", "nbval"],
    install_requires=Path("requirements.txt").read_text().strip().splitlines(),
    include_package_data=True,
    packages=setuptools.find_packages(),
    entry_points={
        "pytest11": ["pytest-pidgy=pidgy.pytest_config"],
        "console_scripts": ["pidgy=pidgy.__main__:application"],
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: IPython",
        "Framework :: Jupyter",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Framework :: Pytest",
    ],
    zip_safe=False,
)

if __name__ == "__main__":
    setuptools.setup(**setup_args)
